@@ -41,7 +41,7 @@ class ReferenceFieldDescriptor(FieldDescriptor):
         value = super().__get__(instance, owner)
 
         if instance is not None and isinstance(value, Document):
-            instance.__fields__[self.name] = value
+            instance.__data__[self.name] = value
 
         return value
 
@@ -59,10 +59,12 @@ class ReferenceField(Field):
     def from_mongo(self, document, value):
         from yadm.documents import Document  # recursive imports
 
-        if isinstance(value, ObjectId):
+        if isinstance(value, (ObjectId, str)):
             if document.__db__ is not None:
                 qs = document.__db__.get_queryset(self.reference_document_class)
-                return qs.with_id(value)
+                doc = qs.with_id(value)
+                return doc if doc else ObjectId(value)
+
             else:
                 return value
 
@@ -88,4 +90,5 @@ class ReferenceField(Field):
             return value.id
 
         else:
-            raise TypeError('value must be ObjectId, Document or dict')
+            raise TypeError('value must be ObjectId, Document or dict,'
+                            ' but {!r}'.format(type(value)))
